from __future__ import unicode_literals

from django import forms
from django.contrib.auth import authenticate, get_user_model
from django.contrib.auth.backends import ModelBackend
from django.utils.translation import ugettext_lazy as _

from .forms import VerifiedEmailField


__all__ = ['EmailAuthenticationForm', 'VerifiedEmailBackend']


class EmailAuthenticationForm(forms.Form):
    """
    Form for authenticating users by verified email.
    """
    email = VerifiedEmailField(_('E-mail'))

    error_messages = {
        'error': _('''User with given e-mail couldn't neither be found nor created.'''),
    }

    def clean(self):
        email = self.cleaned_data.get('email')

        if email:
            self.user = authenticate(verified_email=email)

            if self.user is None:
                raise forms.ValidationError(
                    self.error_messages['error'],
                    code='error',
                )
        return self.cleaned_data

    # following methods make this form be drop in replacement for
    # Django's AuthenticationForm

    def __init__(self, request=None, *args, **kwargs):
        super(EmailAuthenticationForm, self).__init__(*args, **kwargs)

    def get_user_id(self):
        return self.user and self.user.id

    def get_user(self):
        return self.user


class VerifiedEmailBackend(ModelBackend):
    """
    Authenticates against settings.AUTH_USER_MODEL using verified email.
    Fails if there are two active users with given email.
    Fails to create new user if there already is user with given email as username.
    """

    def authenticate(self, verified_email=None, **kwargs):
        if verified_email:
            UserModel = get_user_model()
            try:
                return (
                    UserModel.objects.filter(email=verified_email, is_active=True).last() or
                    UserModel.objects.create_user(username=verified_email, email=verified_email)
                )
            except:
                pass



from __future__ import unicode_literals

from collections import namedtuple

from django import forms


class TextField(forms.CharField):
    widget = forms.Textarea


ReadonlyField = namedtuple('ReadonlyField', ('label', 'value'))



# -*- coding: utf-8 -*-
# Generated by Django 1.9.9 on 2016-11-25 09:15
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import leprikon.models.fields


def populate_registrations(apps, schema_editor):
    for model_name in ['ClubRegistration', 'EventRegistration']:
        Model = apps.get_model('leprikon', model_name)

        for reg in Model.objects.all():
            reg.user = reg.participant.user
            for attr in ['birth_num', 'first_name', 'last_name', 'street', 'city', 'postal_code', 'phone', 'email', 'gender']:
                setattr(reg, 'participant_{}'.format(attr), getattr(reg.participant, attr))
            for n, parent in enumerate(reg.parents.all()[:2]):
                setattr(reg, 'has_parent{}'.format(n+1), True)
                for attr in ['first_name', 'last_name', 'street', 'city', 'postal_code', 'phone', 'email']:
                    setattr(reg, 'parent{}_{}'.format(n+1, attr), getattr(parent, attr))
            reg.save()


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('leprikon', '0018_insurance_cleanup'),
    ]

    operations = [
        migrations.AlterField(
            model_name='clubjournalleaderentry',
            name='timesheet',
            field=models.ForeignKey(editable=False, on_delete=django.db.models.deletion.PROTECT, related_name='club_entries', to='leprikon.Timesheet', verbose_name='timesheet'),
        ),
        migrations.AlterField(
            model_name='timesheetentry',
            name='timesheet',
            field=models.ForeignKey(editable=False, on_delete=django.db.models.deletion.PROTECT, related_name='timesheet_entries', to='leprikon.Timesheet', verbose_name='timesheet'),
        ),
    ]
    for model_name in ['clubregistration', 'eventregistration']:
        for field in ['age_group', 'insurance', 'citizenship', 'school', 'school_other', 'school_class', 'health']:
            operations.append(
                migrations.RenameField(
                    model_name=model_name,
                    old_name=field,
                    new_name='participant_{}'.format(field),
                )
            )
        operations += [
            migrations.AlterField(
                model_name=model_name,
                name='participant_age_group',
                field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='+', to='leprikon.AgeGroup', verbose_name='age group'),
            ),
            migrations.AlterField(
                model_name=model_name,
                name='participant_insurance',
                field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='leprikon.Insurance', verbose_name='insurance'),
            ),
            migrations.AlterField(
                model_name=model_name,
                name='participant_school',
                field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='leprikon.School', verbose_name='school'),
            ),
            migrations.AddField(
                model_name=model_name,
                name='user',
                field=models.ForeignKey(default=None, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='leprikon_{}s'.format(model_name), to=settings.AUTH_USER_MODEL, verbose_name='user'),
                preserve_default=False,
            ),
            migrations.AddField(
                model_name=model_name,
                name='participant_birth_num',
                field=leprikon.models.fields.BirthNumberField(default='', verbose_name='birth number'),
                preserve_default=False,
            ),
            migrations.AddField(
                model_name=model_name,
                name='participant_gender',
                field=models.CharField(choices=[('m', 'male'), ('f', 'female')], default='', editable=False, max_length=1, verbose_name='gender'),
                preserve_default=False,
            ),
            migrations.AddField(
                model_name=model_name,
                name='participant_first_name',
                field=models.CharField(default='', max_length=30, verbose_name='first name'),
                preserve_default=False,
            ),
            migrations.AddField(
                model_name=model_name,
                name='participant_last_name',
                field=models.CharField(default='', max_length=30, verbose_name='last name'),
                preserve_default=False,
            ),
            migrations.AddField(
                model_name=model_name,
                name='participant_street',
                field=models.CharField(default='', max_length=150, verbose_name='street'),
                preserve_default=False,
            ),
            migrations.AddField(
                model_name=model_name,
                name='participant_city',
                field=models.CharField(default='', max_length=150, verbose_name='city'),
                preserve_default=False,
            ),
            migrations.AddField(
                model_name=model_name,
                name='participant_postal_code',
                field=leprikon.models.fields.PostalCodeField(default='', verbose_name='postal code'),
                preserve_default=False,
            ),
            migrations.AddField(
                model_name=model_name,
                name='participant_phone',
                field=models.CharField(blank=True, default='', max_length=30, verbose_name='phone'),
            ),
            migrations.AddField(
                model_name=model_name,
                name='participant_email',
                field=models.EmailField(blank=True, default='', max_length=254, verbose_name='email address'),
            ),
            migrations.AddField(
                model_name=model_name,
                name='has_parent1',
                field=models.BooleanField(default=False, verbose_name='first parent'),
            ),
            migrations.AddField(
                model_name=model_name,
                name='parent1_first_name',
                field=models.CharField(blank=True, max_length=30, null=True, verbose_name='first name'),
            ),
            migrations.AddField(
                model_name=model_name,
                name='parent1_last_name',
                field=models.CharField(blank=True, max_length=30, null=True, verbose_name='last name'),
            ),
            migrations.AddField(
                model_name=model_name,
                name='parent1_street',
                field=models.CharField(blank=True, max_length=150, null=True, verbose_name='street'),
            ),
            migrations.AddField(
                model_name=model_name,
                name='parent1_city',
                field=models.CharField(blank=True, max_length=150, null=True, verbose_name='city'),
            ),
            migrations.AddField(
                model_name=model_name,
                name='parent1_postal_code',
                field=leprikon.models.fields.PostalCodeField(blank=True, null=True, verbose_name='postal code'),
            ),
            migrations.AddField(
                model_name=model_name,
                name='parent1_phone',
                field=models.CharField(blank=True, max_length=30, null=True, verbose_name='phone'),
            ),
            migrations.AddField(
                model_name=model_name,
                name='parent1_email',
                field=models.EmailField(blank=True, max_length=254, null=True, verbose_name='email address'),
            ),
            migrations.AddField(
                model_name=model_name,
                name='has_parent2',
                field=models.BooleanField(default=False, verbose_name='second parent'),
            ),
            migrations.AddField(
                model_name=model_name,
                name='parent2_first_name',
                field=models.CharField(blank=True, max_length=30, null=True, verbose_name='first name'),
            ),
            migrations.AddField(
                model_name=model_name,
                name='parent2_last_name',
                field=models.CharField(blank=True, max_length=30, null=True, verbose_name='last name'),
            ),
            migrations.AddField(
                model_name=model_name,
                name='parent2_street',
                field=models.CharField(blank=True, max_length=150, null=True, verbose_name='street'),
            ),
            migrations.AddField(
                model_name=model_name,
                name='parent2_city',
                field=models.CharField(blank=True, max_length=150, null=True, verbose_name='city'),
            ),
            migrations.AddField(
                model_name=model_name,
                name='parent2_postal_code',
                field=leprikon.models.fields.PostalCodeField(blank=True, null=True, verbose_name='postal code'),
            ),
            migrations.AddField(
                model_name=model_name,
                name='parent2_phone',
                field=models.CharField(blank=True, max_length=30, null=True, verbose_name='phone'),
            ),
            migrations.AddField(
                model_name=model_name,
                name='parent2_email',
                field=models.EmailField(blank=True, max_length=254, null=True, verbose_name='email address'),
            ),
        ]
    operations.append(migrations.RunPython(populate_registrations))
    for model_name in ['clubregistration', 'eventregistration']:
        operations += [
            migrations.AlterField(
                model_name=model_name,
                name='user',
                field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='leprikon_{}s'.format(model_name), to=settings.AUTH_USER_MODEL, verbose_name='user'),
            ),
        ]
    operations += [
        migrations.AlterUniqueTogether(
            name='clubregistration',
            unique_together=set([('club', 'participant_birth_num')]),
        ),
        migrations.AlterUniqueTogether(
            name='eventregistration',
            unique_together=set([('event', 'participant_birth_num')]),
        ),
    ]



from __future__ import unicode_literals

from django import forms
from django.contrib.auth import get_user_model, password_validation
from django.contrib.auth.forms import (
    AuthenticationForm as _AuthenticationForm,
    PasswordChangeForm as _PasswordChangeForm,
    PasswordResetForm as _PasswordResetForm,
    SetPasswordForm as _SetPasswordForm, UserCreationForm as _UserCreationForm,
)
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _

from ..models.roles import Parent
from .form import FormMixin

User = get_user_model()



class UserFormMixin(FormMixin):

    def clean_email(self):
        if self.cleaned_data['email'] and User.objects.filter(email=self.cleaned_data['email']).first():
            raise ValidationError(
                _('User with this email already exists. '
                  'You may use password reset form if You have forgotten your user name or password.'),
                code='exists',
                params={'email': self.cleaned_data['email']},
            )
        else:
            return self.cleaned_data['email']



class UserAdminCreateForm(UserFormMixin, forms.ModelForm):

    # create user with random password
    # users without password may not reset the password themselves
    def save(self, commit=True):
        user = super(UserAdminCreateForm, self).save(commit)
        user.set_password(User.objects.make_random_password())
        user.save()
        return user

    class Meta:
        model = User
        fields = ['username', 'first_name', 'last_name', 'email']




class UserCreateForm(UserFormMixin, _UserCreationForm):

    def __init__(self, **kwargs):
        super(UserCreateForm, self).__init__(**kwargs)
        self.fields['username'].help_text = None
        self.fields['email'].required = True
        self.fields['password1'].help_text = password_validation.password_validators_help_text_html()

    def save(self, commit=True):
        user = super(UserCreateForm, self).save()
        parent = Parent()
        parent.first_name = user.first_name
        parent.last_name = user.last_name
        parent.email = user.email
        parent.user = user
        parent.save()
        return user

    class Meta:
        model = User
        fields = ['username', 'first_name', 'last_name', 'email']



class UserUpdateForm(FormMixin, forms.ModelForm):

    def __init__(self, **kwargs):
        super(UserUpdateForm, self).__init__(**kwargs)
        self.fields['username'].help_text = None

    class Meta:
        model = User
        fields = ['username', 'first_name', 'last_name', 'email']



class UserLoginForm(FormMixin, _AuthenticationForm):
    pass



class UserPasswordForm(FormMixin, _PasswordChangeForm):
    pass



class PasswordResetForm(FormMixin, _PasswordResetForm):
    pass



class SetPasswordForm(FormMixin, _SetPasswordForm):
    pass



# -*- coding: utf-8 -*-
# Generated by Django 1.9.9 on 2016-12-06 17:25
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


def populate_history(apps, schema_editor):
    ClubRegistration = apps.get_model('leprikon', 'clubregistration')
    ClubRegistrationHistory = apps.get_model('leprikon', 'clubregistrationhistory')

    for reg in ClubRegistration.objects.all():
        ClubRegistrationHistory.objects.create(club=reg.club, registration=reg, start=reg.created.date())


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0021_clubjournalentry_participants'),
    ]

    operations = [
        migrations.CreateModel(
            name='clubregistrationhistory',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('start', models.DateField()),
                ('end', models.DateField(default=None, null=True)),
                ('club', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='registrations_history', to='leprikon.Club', verbose_name='club')),
                ('registration', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='club_history', to='leprikon.ClubRegistration', verbose_name='club')),
            ],
            options={'ordering': ('start',)},
        ),
        migrations.RunPython(populate_history),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.9 on 2016-11-10 21:21
from __future__ import unicode_literals

from django.db import migrations, models


def set_gender(apps, schema_editor):

    Participant = apps.get_model('leprikon', 'Participant')

    for participant in Participant.objects.all():
        if participant.birth_num[2:4] > '50':
            participant.gender = 'f'
            participant.save()


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0013_allow_duplicit_participants'),
    ]

    operations = [
        migrations.AddField(
            model_name='participant',
            name='gender',
            field=models.CharField(choices=[('m', 'male'), ('f', 'female')], default='m', editable=False, max_length=1, verbose_name='gender'),
        ),
        migrations.RunPython(set_gender),
        migrations.AlterField(
            model_name='participant',
            name='gender',
            field=models.CharField(choices=[('m', 'male'), ('f', 'female')], editable=False, max_length=1, verbose_name='gender'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.13 on 2018-01-16 13:14
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0007_bill_print_setup'),
    ]

    operations = [
        migrations.AddField(
            model_name='leprikonsite',
            name='company_name',
            field=models.CharField(blank=True, max_length=150, null=True, verbose_name='company name'),
        ),
    ]



from __future__ import unicode_literals

from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.functional import cached_property
from django.utils.translation import ugettext_lazy as _


@python_2_unicode_compatible
class School(models.Model):
    name    = models.CharField(_('name'),   max_length=50)
    street  = models.CharField(_('street'), max_length=50, blank=True, default='')
    city    = models.CharField(_('city'),   max_length=50, blank=True, default='')

    class Meta:
        app_label           = 'leprikon'
        ordering            = ('city', 'name')
        verbose_name        = _('school')
        verbose_name_plural = _('schools')

    def __str__(self):
        if self.address:
            return '{}, {}'.format(self.name, self.address)
        else:
            return self.name

    @cached_property
    def address(self):
        if self.street and self.city:
            return '{}, {}'.format(self.street, self.city)
        else:
            return self.street or self.city



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-06-03 06:18
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


def fix_message_dates(apps, schema_editor):
    from datetime import date, timedelta

    MessageRecipient = apps.get_model('leprikon', 'MessageRecipient')

    for rec in MessageRecipient.objects.all():
        rec.sent_mail = rec.sent
        rec.save()


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0006_deletable_pages'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='message',
            options={'ordering': ('-created',), 'verbose_name': 'message', 'verbose_name_plural': 'messages'},
        ),
        migrations.AlterModelOptions(
            name='messagerecipient',
            options={'ordering': ('sent',), 'verbose_name': 'recipient', 'verbose_name_plural': 'recipients'},
        ),
        migrations.RenameField(
            model_name='message',
            old_name='sent',
            new_name='created',
        ),
        migrations.AlterField(
            model_name='message',
            name='created',
            field=models.DateTimeField(auto_now_add=True, verbose_name='created'),
        ),
        migrations.AlterField(
            model_name='messagerecipient',
            name='message',
            field=models.ForeignKey(editable=False, on_delete=django.db.models.deletion.CASCADE, related_name='recipients', to='leprikon.Message', verbose_name='message'),
        ),
        migrations.AlterField(
            model_name='messagerecipient',
            name='recipient',
            field=models.ForeignKey(editable=False, on_delete=django.db.models.deletion.CASCADE, related_name='leprikon_messages', to=settings.AUTH_USER_MODEL, verbose_name='recipient'),
        ),
        migrations.RenameField(
            model_name='messagerecipient',
            old_name='received',
            new_name='viewed',
        ),
        migrations.AlterField(
            model_name='messagerecipient',
            name='viewed',
            field=models.DateTimeField(default=None, editable=False, null=True, verbose_name='viewed on site'),
        ),
        migrations.AddField(
            model_name='messagerecipient',
            name='sent_mail',
            field=models.DateTimeField(default=None, editable=False, null=True, verbose_name='sent by email'),
        ),
        migrations.RunPython(fix_message_dates),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.4 on 2016-03-26 11:09
from __future__ import unicode_literals

import cms.models.fields
from django.db import migrations, models
from leprikon.conf import settings
import django.db.models.deletion
import djangocms_text_ckeditor.fields
import filer.fields.file
import filer.fields.image
import leprikon.models.courses
import leprikon.models.fields
import leprikon.models.startend


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('cms', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('filer', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='AgeGroup',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50, verbose_name='name')),
                ('order', models.IntegerField(blank=True, default=0, verbose_name='order')),
            ],
            options={
                'ordering': ('order',),
                'verbose_name': 'age group',
                'verbose_name_plural': 'age groups',
            },
        ),
        migrations.CreateModel(
            name='Club',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=150, verbose_name='name')),
                ('description', djangocms_text_ckeditor.fields.HTMLField(blank=True, default='', verbose_name='description')),
                ('price', leprikon.models.fields.PriceField(decimal_places=0, max_digits=10, verbose_name='price')),
                ('unit', models.CharField(max_length=150, verbose_name='unit')),
                ('public', models.BooleanField(default=False, verbose_name='public')),
                ('reg_active', models.BooleanField(default=False, verbose_name='active registration')),
                ('min_count', models.IntegerField(blank=True, null=True, verbose_name='minimal count')),
                ('max_count', models.IntegerField(blank=True, null=True, verbose_name='maximal count')),
                ('risks', djangocms_text_ckeditor.fields.HTMLField(blank=True, verbose_name='risks')),
                ('plan', djangocms_text_ckeditor.fields.HTMLField(blank=True, verbose_name='plan')),
                ('evaluation', djangocms_text_ckeditor.fields.HTMLField(blank=True, verbose_name='evaluation')),
                ('note', models.CharField(blank=True, default='', max_length=300, verbose_name='note')),
                ('age_groups', models.ManyToManyField(blank=True, related_name='clubs', to='leprikon.AgeGroup', verbose_name='age groups')),
            ],
            options={
                'ordering': ('name',),
                'verbose_name': 'club',
                'verbose_name_plural': 'clubs',
            },
        ),
        migrations.CreateModel(
            name='ClubAttachment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('order', models.IntegerField(blank=True, default=0, verbose_name='order')),
                ('club', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='attachments', to='leprikon.Club', verbose_name='club')),
                ('file', filer.fields.file.FilerFileField(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='filer.File')),
            ],
            options={
                'ordering': ('order',),
                'verbose_name': 'attachment',
                'verbose_name_plural': 'attachments',
            },
        ),
        migrations.CreateModel(
            name='ClubGroup',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=150, verbose_name='name')),
                ('plural', models.CharField(max_length=150, verbose_name='plural')),
                ('color', leprikon.models.fields.ColorField(max_length=10, verbose_name='color')),
                ('order', models.IntegerField(blank=True, default=0, verbose_name='order')),
            ],
            options={
                'ordering': ('order',),
                'verbose_name': 'club group',
                'verbose_name_plural': 'club groups',
            },
        ),
        migrations.CreateModel(
            name='ClubJournalEntry',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date', models.DateField(verbose_name='date')),
                ('start', models.TimeField(blank=True, help_text='Leave empty, if the club does not take place', null=True, verbose_name='start time')),
                ('end', models.TimeField(blank=True, help_text='Leave empty, if the club does not take place', null=True, verbose_name='end time')),
                ('agenda', djangocms_text_ckeditor.fields.HTMLField(default=leprikon.models.courses.get_default_agenda, verbose_name='session agenda')),
                ('club', models.ForeignKey(editable=False, on_delete=django.db.models.deletion.CASCADE, related_name='journal_entries', to='leprikon.Club', verbose_name='club')),
            ],
            options={
                'ordering': ('date', 'start', 'end'),
                'verbose_name': 'journal entry',
                'verbose_name_plural': 'journal entries',
            },
            bases=(leprikon.models.startend.StartEndMixin, models.Model),
        ),
        migrations.CreateModel(
            name='ClubJournalLeaderEntry',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('start', models.TimeField(verbose_name='start time')),
                ('end', models.TimeField(verbose_name='end time')),
                ('club_entry', models.ForeignKey(editable=False, on_delete=django.db.models.deletion.CASCADE, related_name='leader_entries', to='leprikon.ClubJournalEntry', verbose_name='club journal entry')),
            ],
            options={
                'verbose_name': 'club journal leader entry',
                'verbose_name_plural': 'club journal leader entries',
            },
            bases=(leprikon.models.startend.StartEndMixin, models.Model),
        ),
        migrations.CreateModel(
            name='ClubPayment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date', models.DateField(verbose_name='payment date')),
                ('amount', leprikon.models.fields.PriceField(decimal_places=0, max_digits=10, verbose_name='amount')),
            ],
            options={
                'verbose_name': 'club payment',
                'verbose_name_plural': 'club payments',
            },
        ),
        migrations.CreateModel(
            name='ClubPeriod',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=150, verbose_name='name')),
                ('start', models.DateField(verbose_name='start date')),
                ('end', models.DateField(verbose_name='end date')),
                ('club', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='periods', to='leprikon.Club', verbose_name='club')),
            ],
            options={
                'ordering': ('club__name', 'start'),
                'verbose_name': 'period',
                'verbose_name_plural': 'periods',
            },
            bases=(leprikon.models.startend.StartEndMixin, models.Model),
        ),
        migrations.CreateModel(
            name='ClubRegistration',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.SlugField(editable=False)),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='time of registration')),
                ('citizenship', models.CharField(max_length=50, verbose_name='citizenship')),
                ('insurance', models.CharField(max_length=50, verbose_name='insurance')),
                ('school_other', models.CharField(blank=True, default='', max_length=150, verbose_name='other school')),
                ('school_class', models.CharField(blank=True, default='', max_length=30, verbose_name='class')),
                ('health', models.TextField(blank=True, default='', verbose_name='health')),
                ('answers', models.TextField(blank=True, default='{}', editable=False, verbose_name='additional answers')),
                ('cancel_request', models.BooleanField(default=False, verbose_name='cancel request')),
                ('canceled', models.DateField(blank=True, null=True, verbose_name='date of cancellation')),
                ('discount', leprikon.models.fields.PriceField(decimal_places=0, default=0, max_digits=10, verbose_name='discount')),
                ('explanation', models.TextField(blank=True, default='', verbose_name='discount explanation')),
                ('age_group', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='leprikon.AgeGroup', verbose_name='age group')),
                ('club', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='registrations', to='leprikon.Club', verbose_name='club')),
            ],
            options={
                'verbose_name': 'club registration',
                'verbose_name_plural': 'club registrations',
            },
        ),
        migrations.CreateModel(
            name='ClubTime',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('day_of_week', leprikon.models.fields.DayOfWeekField(choices=[(1, 'Monday'), (2, 'Tuesday'), (3, 'Wednesday'), (4, 'Thursday'), (5, 'Friday'), (6, 'Saturday'), (7, 'Sunday')], verbose_name='day of week')),
                ('start', models.TimeField(blank=True, null=True, verbose_name='start time')),
                ('end', models.TimeField(blank=True, null=True, verbose_name='end time')),
                ('club', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='times', to='leprikon.Club', verbose_name='club')),
            ],
            options={
                'ordering': ('day_of_week', 'start'),
                'verbose_name': 'time',
                'verbose_name_plural': 'times',
            },
            bases=(leprikon.models.startend.StartEndMixin, models.Model),
        ),
        migrations.CreateModel(
            name='Contact',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('contact_type', models.CharField(choices=[('email', 'email'), ('phone', 'phone'), ('url', 'url')], max_length=30, verbose_name='contact type')),
                ('contact', models.CharField(max_length=250, verbose_name='contact')),
                ('order', models.IntegerField(blank=True, default=0, verbose_name='order')),
                ('public', models.BooleanField(default=False, verbose_name='public')),
            ],
            options={
                'ordering': ('order',),
                'verbose_name': 'contact',
                'verbose_name_plural': 'contacts',
            },
        ),
        migrations.CreateModel(
            name='Event',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=150, verbose_name='name')),
                ('description', djangocms_text_ckeditor.fields.HTMLField(blank=True, default='', verbose_name='description')),
                ('start_date', models.DateField(verbose_name='start date')),
                ('end_date', models.DateField(verbose_name='end date')),
                ('start_time', models.TimeField(blank=True, null=True, verbose_name='start time')),
                ('end_time', models.TimeField(blank=True, null=True, verbose_name='end time')),
                ('price', leprikon.models.fields.PriceField(decimal_places=0, max_digits=10, verbose_name='price')),
                ('public', models.BooleanField(default=False, verbose_name='public')),
                ('reg_active', models.BooleanField(default=False, verbose_name='active registration')),
                ('min_count', models.IntegerField(blank=True, null=True, verbose_name='minimal count')),
                ('max_count', models.IntegerField(blank=True, null=True, verbose_name='maximal count')),
                ('risks', djangocms_text_ckeditor.fields.HTMLField(blank=True, verbose_name='risks')),
                ('plan', djangocms_text_ckeditor.fields.HTMLField(blank=True, verbose_name='plan')),
                ('evaluation', djangocms_text_ckeditor.fields.HTMLField(blank=True, verbose_name='evaluation')),
                ('note', models.CharField(blank=True, default='', max_length=300, verbose_name='note')),
                ('age_groups', models.ManyToManyField(blank=True, related_name='events', to='leprikon.AgeGroup', verbose_name='age groups')),
            ],
            options={
                'ordering': ('start_date', 'start_time'),
                'verbose_name': 'event',
                'verbose_name_plural': 'events',
            },
        ),
        migrations.CreateModel(
            name='EventAttachment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('order', models.IntegerField(blank=True, default=0, verbose_name='order')),
                ('event', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='attachments', to='leprikon.Event', verbose_name='event')),
                ('file', filer.fields.file.FilerFileField(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='filer.File')),
            ],
            options={
                'ordering': ('order',),
                'verbose_name': 'attachment',
                'verbose_name_plural': 'attachments',
            },
        ),
        migrations.CreateModel(
            name='EventGroup',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=150, verbose_name='name')),
                ('plural', models.CharField(max_length=150, verbose_name='plural')),
                ('color', leprikon.models.fields.ColorField(max_length=10, verbose_name='color')),
                ('order', models.IntegerField(blank=True, default=0, verbose_name='order')),
            ],
            options={
                'ordering': ('order',),
                'verbose_name': 'event group',
                'verbose_name_plural': 'event groups',
            },
        ),
        migrations.CreateModel(
            name='EventPayment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date', models.DateField(verbose_name='payment date')),
                ('amount', leprikon.models.fields.PriceField(decimal_places=0, max_digits=10, verbose_name='amount')),
            ],
            options={
                'verbose_name': 'event payment',
                'verbose_name_plural': 'event payments',
            },
        ),
        migrations.CreateModel(
            name='EventRegistration',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.SlugField(editable=False)),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='time of registration')),
                ('citizenship', models.CharField(max_length=50, verbose_name='citizenship')),
                ('insurance', models.CharField(max_length=50, verbose_name='insurance')),
                ('school_other', models.CharField(blank=True, default='', max_length=150, verbose_name='other school')),
                ('school_class', models.CharField(blank=True, default='', max_length=30, verbose_name='class')),
                ('health', models.TextField(blank=True, default='', verbose_name='health')),
                ('answers', models.TextField(blank=True, default='{}', editable=False, verbose_name='additional answers')),
                ('cancel_request', models.BooleanField(default=False, verbose_name='cancel request')),
                ('canceled', models.DateField(blank=True, null=True, verbose_name='date of cancellation')),
                ('discount', leprikon.models.fields.PriceField(decimal_places=0, default=0, max_digits=10, verbose_name='discount')),
                ('explanation', models.TextField(blank=True, default='', verbose_name='discount explanation')),
                ('age_group', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='leprikon.AgeGroup', verbose_name='age group')),
                ('event', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='registrations', to='leprikon.Event', verbose_name='event')),
            ],
            options={
                'verbose_name': 'event registration',
                'verbose_name_plural': 'event registrations',
            },
        ),
        migrations.CreateModel(
            name='EventType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.SlugField()),
                ('name', models.CharField(max_length=150, verbose_name='name')),
                ('order', models.IntegerField(blank=True, default=0, verbose_name='order')),
            ],
            options={
                'ordering': ('order',),
                'verbose_name': 'event type',
                'verbose_name_plural': 'event types',
            },
        ),
        migrations.CreateModel(
            name='EventTypeAttachment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('order', models.IntegerField(blank=True, default=0, verbose_name='order')),
                ('event', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='attachments', to='leprikon.EventType', verbose_name='event type')),
                ('file', filer.fields.file.FilerFileField(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='filer.File')),
            ],
            options={
                'ordering': ('order',),
                'verbose_name': 'attachment',
                'verbose_name_plural': 'attachments',
            },
        ),
        migrations.CreateModel(
            name='Leader',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('description', djangocms_text_ckeditor.fields.HTMLField(blank=True, default='', verbose_name='description')),
                ('page', cms.models.fields.PageField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='cms.Page', verbose_name='page')),
                ('photo', filer.fields.image.FilerImageField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='filer.Image', verbose_name='photo')),
            ],
            options={
                'ordering': ('user__first_name', 'user__last_name'),
                'verbose_name': 'leader',
                'verbose_name_plural': 'leaders',
            },
        ),
        migrations.CreateModel(
            name='LeprikonClubListPlugin',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='cms.CMSPlugin')),
                ('template', models.CharField(choices=[('default', 'Default'), ('grouped', 'Grouped by club groups')], default='default', help_text='The template used to render plugin.', max_length=100, verbose_name='template')),
                ('age_groups', models.ManyToManyField(blank=True, help_text='Keep empty to skip searching by age groups.', to='leprikon.AgeGroup', verbose_name='age groups')),
                ('groups', models.ManyToManyField(blank=True, help_text='Keep empty to skip searching by groups.', to='leprikon.ClubGroup', verbose_name='club groups')),
                ('leaders', models.ManyToManyField(blank=True, help_text='Keep empty to skip searching by leaders.', to='leprikon.Leader', verbose_name='leaders')),
            ],
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='LeprikonEventListPlugin',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='cms.CMSPlugin')),
                ('template', models.CharField(choices=[('default', 'Default'), ('grouped', 'Grouped by event groups')], default='default', help_text='The template used to render plugin.', max_length=100, verbose_name='template')),
                ('age_groups', models.ManyToManyField(blank=True, help_text='Keep empty to skip searching by age groups.', to='leprikon.AgeGroup', verbose_name='age groups')),
                ('event_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='leprikon.EventType', verbose_name='event type')),
                ('groups', models.ManyToManyField(blank=True, help_text='Keep empty to skip searching by groups.', to='leprikon.EventGroup', verbose_name='event groups')),
                ('leaders', models.ManyToManyField(blank=True, help_text='Keep empty to skip searching by leaders.', to='leprikon.Leader', verbose_name='leaders')),
            ],
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='LeprikonFilteredClubListPlugin',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='cms.CMSPlugin')),
            ],
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='LeprikonFilteredEventListPlugin',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='cms.CMSPlugin')),
                ('event_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='leprikon.EventType', verbose_name='event type')),
            ],
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='Parent',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('first_name', models.CharField(max_length=30, verbose_name='first name')),
                ('last_name', models.CharField(max_length=30, verbose_name='last name')),
                ('street', models.CharField(max_length=150, verbose_name='street')),
                ('city', models.CharField(max_length=150, verbose_name='city')),
                ('postal_code', leprikon.models.fields.PostalCodeField(verbose_name='postal code')),
                ('email', models.EmailField(blank=True, default='', max_length=254, verbose_name='email address')),
                ('phone', models.CharField(max_length=30, verbose_name='phone')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='leprikon_parents', to=settings.AUTH_USER_MODEL, verbose_name='user')),
            ],
            options={
                'verbose_name': 'parent',
                'verbose_name_plural': 'parents',
            },
        ),
        migrations.CreateModel(
            name='Participant',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('first_name', models.CharField(max_length=30, verbose_name='first name')),
                ('last_name', models.CharField(max_length=30, verbose_name='last name')),
                ('birth_num', leprikon.models.fields.BirthNumberField(unique=True, verbose_name='birth number')),
                ('street', models.CharField(max_length=150, verbose_name='street')),
                ('city', models.CharField(max_length=150, verbose_name='city')),
                ('postal_code', leprikon.models.fields.PostalCodeField(verbose_name='postal code')),
                ('email', models.EmailField(blank=True, default='', max_length=254, verbose_name='email address')),
                ('phone', models.CharField(blank=True, default='', max_length=30, verbose_name='phone')),
                ('citizenship', models.CharField(max_length=50, verbose_name='citizenship')),
                ('insurance', models.CharField(max_length=50, verbose_name='insurance')),
                ('school_other', models.CharField(blank=True, default='', max_length=150, verbose_name='other school')),
                ('school_class', models.CharField(blank=True, default='', max_length=30, verbose_name='class')),
                ('health', models.TextField(blank=True, default='', verbose_name='health')),
                ('age_group', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='leprikon.AgeGroup', verbose_name='age group')),
                ('parents', models.ManyToManyField(blank=True, related_name='participants', to='leprikon.Parent', verbose_name='parents')),
            ],
            options={
                'verbose_name': 'participant',
                'verbose_name_plural': 'participants',
            },
        ),
        migrations.CreateModel(
            name='Place',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50, verbose_name='name')),
                ('place', models.CharField(blank=True, default='', max_length=50, verbose_name='place')),
            ],
            options={
                'ordering': ('place',),
                'verbose_name': 'place',
                'verbose_name_plural': 'places',
            },
        ),
        migrations.CreateModel(
            name='Question',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50, unique=True, verbose_name='name')),
                ('question', models.CharField(max_length=50, verbose_name='question')),
                ('help_text', models.TextField(blank=True, help_text='This is help text. The help text is shown next to the form field.', null=True, verbose_name='help text')),
                ('field', models.CharField(choices=((key, val['name']) for key, val in settings.LEPRIKON_QUESTION_FIELDS.items()), max_length=150, verbose_name='field')),
                ('field_args', models.TextField(blank=True, default='{}', help_text='Enter valid JSON structure representing field configuration.', verbose_name='field_args')),
            ],
            options={
                'verbose_name': 'additional question',
                'verbose_name_plural': 'additional questions',
            },
        ),
        migrations.CreateModel(
            name='School',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50, verbose_name='name')),
                ('street', models.CharField(blank=True, default='', max_length=50, verbose_name='street')),
                ('city', models.CharField(blank=True, default='', max_length=50, verbose_name='city')),
            ],
            options={
                'ordering': ('city', 'name'),
                'verbose_name': 'school',
                'verbose_name_plural': 'schools',
            },
        ),
        migrations.CreateModel(
            name='SchoolYear',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('year', models.IntegerField(unique=True, verbose_name='year')),
                ('active', models.BooleanField(default=False, verbose_name='active')),
            ],
            options={
                'ordering': ('-year',),
                'verbose_name': 'school year',
                'verbose_name_plural': 'school years',
            },
        ),
        migrations.CreateModel(
            name='Timesheet',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('submitted', models.BooleanField(default=False, verbose_name='submitted')),
                ('paid', models.BooleanField(default=False, verbose_name='paid')),
                ('leader', models.ForeignKey(editable=False, on_delete=django.db.models.deletion.CASCADE, related_name='timesheets', to='leprikon.Leader', verbose_name='leader')),
            ],
            options={
                'ordering': ('-period__start',),
                'verbose_name': 'timesheet',
                'verbose_name_plural': 'timesheets',
            },
        ),
        migrations.CreateModel(
            name='TimesheetEntry',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date', models.DateField(verbose_name='date')),
                ('start', models.TimeField(verbose_name='start time')),
                ('end', models.TimeField(verbose_name='end time')),
                ('description', djangocms_text_ckeditor.fields.HTMLField(verbose_name='work description')),
            ],
            options={
                'ordering': ('start',),
                'verbose_name': 'timesheet entry',
                'verbose_name_plural': 'timesheet entries',
            },
            bases=(leprikon.models.startend.StartEndMixin, models.Model),
        ),
        migrations.CreateModel(
            name='TimesheetEntryType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=150, verbose_name='name')),
                ('order', models.IntegerField(blank=True, default=0, verbose_name='order')),
            ],
            options={
                'ordering': ('order',),
                'verbose_name': 'timesheet entry type',
                'verbose_name_plural': 'timesheet entry types',
            },
        ),
        migrations.CreateModel(
            name='TimesheetPeriod',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('start', models.DateField(editable=False, unique=True, verbose_name='start date')),
                ('end', models.DateField(editable=False, unique=True, verbose_name='end date')),
            ],
            options={
                'ordering': ('-start',),
                'verbose_name': 'timesheet period',
                'verbose_name_plural': 'timesheet periods',
            },
            bases=(leprikon.models.startend.StartEndMixin, models.Model),
        ),
        migrations.AddField(
            model_name='timesheetentry',
            name='entry_type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='entries', to='leprikon.TimesheetEntryType', verbose_name='entry type'),
        ),
        migrations.AddField(
            model_name='timesheetentry',
            name='timesheet',
            field=models.ForeignKey(editable=False, on_delete=django.db.models.deletion.CASCADE, related_name='timesheet_entries', to='leprikon.Timesheet', verbose_name='timesheet'),
        ),
        migrations.AddField(
            model_name='timesheet',
            name='period',
            field=models.ForeignKey(editable=False, on_delete=django.db.models.deletion.CASCADE, related_name='timesheets', to='leprikon.TimesheetPeriod', verbose_name='period'),
        ),
        migrations.AddField(
            model_name='participant',
            name='school',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='participants', to='leprikon.School', verbose_name='school'),
        ),
        migrations.AddField(
            model_name='participant',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='leprikon_participants', to=settings.AUTH_USER_MODEL, verbose_name='user'),
        ),
        migrations.AddField(
            model_name='leprikonfilteredeventlistplugin',
            name='school_year',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='leprikon.SchoolYear', verbose_name='school year'),
        ),
        migrations.AddField(
            model_name='leprikonfilteredclublistplugin',
            name='school_year',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='leprikon.SchoolYear', verbose_name='school year'),
        ),
        migrations.AddField(
            model_name='leprikoneventlistplugin',
            name='school_year',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='leprikon.SchoolYear', verbose_name='school year'),
        ),
        migrations.AddField(
            model_name='leprikonclublistplugin',
            name='school_year',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='leprikon.SchoolYear', verbose_name='school year'),
        ),
        migrations.AddField(
            model_name='leader',
            name='school_years',
            field=models.ManyToManyField(related_name='leaders', to='leprikon.SchoolYear', verbose_name='school years'),
        ),
        migrations.AddField(
            model_name='leader',
            name='user',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='leprikon_leader', to=settings.AUTH_USER_MODEL, verbose_name='user'),
        ),
        migrations.AddField(
            model_name='eventtype',
            name='questions',
            field=models.ManyToManyField(blank=True, help_text='Add additional questions to be asked in the registration form.', to='leprikon.Question', verbose_name='additional questions'),
        ),
        migrations.AddField(
            model_name='eventregistration',
            name='participant',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='event_registrations', to='leprikon.Participant', verbose_name='participant'),
        ),
        migrations.AddField(
            model_name='eventregistration',
            name='school',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='event_registrations', to='leprikon.School', verbose_name='school'),
        ),
        migrations.AddField(
            model_name='eventpayment',
            name='registration',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='payments', to='leprikon.EventRegistration', verbose_name='registration'),
        ),
        migrations.AddField(
            model_name='event',
            name='event_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='events', to='leprikon.EventType', verbose_name='event type'),
        ),
        migrations.AddField(
            model_name='event',
            name='groups',
            field=models.ManyToManyField(related_name='events', to='leprikon.EventGroup', verbose_name='groups'),
        ),
        migrations.AddField(
            model_name='event',
            name='leaders',
            field=models.ManyToManyField(blank=True, related_name='events', to='leprikon.Leader', verbose_name='leaders'),
        ),
        migrations.AddField(
            model_name='event',
            name='page',
            field=cms.models.fields.PageField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='cms.Page', verbose_name='page'),
        ),
        migrations.AddField(
            model_name='event',
            name='photo',
            field=filer.fields.image.FilerImageField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='filer.Image', verbose_name='photo'),
        ),
        migrations.AddField(
            model_name='event',
            name='place',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='events', to='leprikon.Place', verbose_name='place'),
        ),
        migrations.AddField(
            model_name='event',
            name='questions',
            field=models.ManyToManyField(blank=True, help_text='Add additional questions to be asked in the registration form.', to='leprikon.Question', verbose_name='additional questions'),
        ),
        migrations.AddField(
            model_name='event',
            name='school_year',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='events', to='leprikon.SchoolYear', verbose_name='school year'),
        ),
        migrations.AddField(
            model_name='contact',
            name='leader',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='contacts', to='leprikon.Leader', verbose_name='leader'),
        ),
        migrations.AddField(
            model_name='clubregistration',
            name='participant',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='club_registrations', to='leprikon.Participant', verbose_name='participant'),
        ),
        migrations.AddField(
            model_name='clubregistration',
            name='school',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='club_registrations', to='leprikon.School', verbose_name='school'),
        ),
        migrations.AddField(
            model_name='clubpayment',
            name='registration',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='payments', to='leprikon.ClubRegistration', verbose_name='registration'),
        ),
        migrations.AddField(
            model_name='clubjournalleaderentry',
            name='timesheet',
            field=models.ForeignKey(editable=False, on_delete=django.db.models.deletion.CASCADE, related_name='club_entries', to='leprikon.Timesheet', verbose_name='timesheet'),
        ),
        migrations.AddField(
            model_name='clubjournalentry',
            name='participants',
            field=models.ManyToManyField(blank=True, related_name='journal_entries', to='leprikon.Participant', verbose_name='participants'),
        ),
        migrations.AddField(
            model_name='club',
            name='groups',
            field=models.ManyToManyField(related_name='clubs', to='leprikon.ClubGroup', verbose_name='groups'),
        ),
        migrations.AddField(
            model_name='club',
            name='leaders',
            field=models.ManyToManyField(blank=True, related_name='clubs', to='leprikon.Leader', verbose_name='leaders'),
        ),
        migrations.AddField(
            model_name='club',
            name='page',
            field=cms.models.fields.PageField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='cms.Page', verbose_name='page'),
        ),
        migrations.AddField(
            model_name='club',
            name='photo',
            field=filer.fields.image.FilerImageField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='filer.Image', verbose_name='photo'),
        ),
        migrations.AddField(
            model_name='club',
            name='place',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='clubs', to='leprikon.Place', verbose_name='place'),
        ),
        migrations.AddField(
            model_name='club',
            name='questions',
            field=models.ManyToManyField(blank=True, help_text='Add additional questions to be asked in the registration form.', to='leprikon.Question', verbose_name='additional questions'),
        ),
        migrations.AddField(
            model_name='club',
            name='school_year',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='clubs', to='leprikon.SchoolYear', verbose_name='school year'),
        ),
        migrations.AlterUniqueTogether(
            name='timesheet',
            unique_together=set([('period', 'leader')]),
        ),
        migrations.AlterUniqueTogether(
            name='eventregistration',
            unique_together=set([('event', 'participant')]),
        ),
        migrations.AlterUniqueTogether(
            name='clubregistration',
            unique_together=set([('club', 'participant')]),
        ),
        migrations.AlterUniqueTogether(
            name='clubjournalleaderentry',
            unique_together=set([('club_entry', 'timesheet')]),
        ),
    ]



from __future__ import unicode_literals

from django.core.urlresolvers import reverse_lazy as reverse
from django.http import HttpResponse
from django.shortcuts import get_object_or_404, redirect
from django.utils.translation import ugettext_lazy as _

from ..forms.subjects import (
    CourseRegistrationForm, EventRegistrationForm, SubjectFilterForm,
    SubjectForm,
)
from ..models.courses import Course
from ..models.events import Event
from ..models.subjects import (
    Subject, SubjectPayment, SubjectRegistration, SubjectType,
)
from .generic import (
    ConfirmUpdateView, CreateView, DetailView, FilteredListView, ListView,
    UpdateView,
)


class SubjectTypeMixin(object):
    _models = {
        SubjectType.COURSE: Course,
        SubjectType.EVENT:  Event,
    }

    def get_template_names(self):
        return [
            'leprikon/{}{}.html'.format(self.subject_type.slug, self.template_name_suffix),
            'leprikon/{}{}.html'.format(self.subject_type.subject_type, self.template_name_suffix),
            'leprikon/subject{}.html'.format(self.template_name_suffix),
        ]

    def dispatch(self, request, subject_type, *args, **kwargs):
        self.subject_type = get_object_or_404(SubjectType, slug=subject_type)
        self.model = self._models[self.subject_type.subject_type]
        return super(SubjectTypeMixin, self).dispatch(request, **kwargs)

    def get_queryset(self):
        return super(SubjectTypeMixin, self).get_queryset().filter(subject_type=self.subject_type)



class SubjectListView(SubjectTypeMixin, FilteredListView):
    form_class          = SubjectFilterForm
    preview_template    = 'leprikon/subject_preview.html'
    paginate_by         = 10

    def get_title(self):
        return _('{subject_type} in school year {school_year}').format(
            subject_type    = self.subject_type.plural,
            school_year     = self.request.school_year,
        )

    def get_message_empty(self):
        return _('No {subject_type} matching given search parameters found.').format(
            subject_type = self.subject_type.plural,
        )

    def get_form(self):
        return self.form_class(
            subject_type_type   = self.subject_type.subject_type,
            subject_types       = [self.subject_type],
            school_year         = self.request.school_year,
            is_staff            = self.request.user.is_staff,
            data                = self.request.GET,
        )

    def get_queryset(self):
        return self.get_form().get_queryset()



class SubjectListMineView(SubjectListView):

    def get_template_names(self):
        return [
            'leprikon/{}_list_mine.html'.format(self.subject_type.slug),
            'leprikon/{}_list_mine.html'.format(self.subject_type.subject_type),
            'leprikon/subject_list_mine.html',
        ] + super(SubjectListMineView, self).get_template_names()

    def get_title(self):
        return _('My {subject_type} in school year {school_year}').format(
            subject_type = self.subject_type.plural,
            school_year = self.request.school_year,
        )

    def get_queryset(self):
        return super(SubjectListMineView, self).get_queryset().filter(leaders=self.request.leader)



class SubjectDetailView(SubjectTypeMixin, DetailView):

    def get_queryset(self):
        qs = super(SubjectDetailView, self).get_queryset()
        if not self.request.user.is_staff:
            qs = qs.filter(public=True)
        return qs



class SubjectRegistrationsView(SubjectTypeMixin, DetailView):
    template_name_suffix = '_registrations'

    def get_queryset(self):
        qs = super(SubjectRegistrationsView, self).get_queryset()
        if not self.request.user.is_staff:
            qs = qs.filter(leaders=self.request.leader)
        return qs



class SubjectUpdateView(SubjectTypeMixin, UpdateView):
    form_class  = SubjectForm

    def get_title(self):
        return _('Change {subject_type} {subject}').format(
            subject_type = self.subject_type.name_akuzativ,
            subject = self.object.name,
        )

    def get_message(self):
        return _('The changes in {subject_type} {subject} have been saved.').format(
            subject_type = self.subject_type.name_genitiv,
            subject = self.object.name,
        )

    def get_queryset(self):
        qs = super(SubjectUpdateView, self).get_queryset()
        if not self.request.user.is_staff:
            qs = qs.filter(leaders=self.request.leader)
        return qs



class SubjectRegistrationFormView(CreateView):
    back_url        = reverse('leprikon:registration_list')
    submit_label    = _('Submit registration')
    message         = _('The registration has been accepted.')
    _models = {
        SubjectType.COURSE: Course,
        SubjectType.EVENT:  Event,
    }
    _form_classes = {
        SubjectType.COURSE: CourseRegistrationForm,
        SubjectType.EVENT:  EventRegistrationForm,
    }

    def get_template_names(self):
        return [
            'leprikon/{}_registration_form.html'.format(self.subject_type.slug, self.template_name_suffix),
            'leprikon/{}_registration_form.html'.format(self.subject_type.subject_type, self.template_name_suffix),
            'leprikon/subject_registration_form.html'.format(self.template_name_suffix),
        ]

    def get_title(self):
        return _('Registration for {subject_type} {subject}').format(
            subject_type = self.subject_type.name_akuzativ,
            subject = self.subject.name,
        )

    def dispatch(self, request, subject_type, pk, **kwargs):
        self.subject_type = get_object_or_404(SubjectType, slug=subject_type)
        lookup_kwargs = {
            'subject_type': self.subject_type,
            'id':           int(pk),
        }
        if not self.request.user.is_staff:
            lookup_kwargs['public'] = True
        self.subject = get_object_or_404(Subject, **lookup_kwargs)
        if not self.subject.registration_allowed:
            return redirect(self.subject)
        self.request.school_year = self.subject.school_year
        self.model = self._models[self.subject.subject_type.subject_type]
        return super(SubjectRegistrationFormView, self).dispatch(request, **kwargs)

    def get_form_class(self):
        return self._form_classes[self.subject.subject_type.subject_type]

    def get_form_kwargs(self):
        kwargs  = super(SubjectRegistrationFormView, self).get_form_kwargs()
        kwargs['subject'] = self.subject
        kwargs['user'] = self.request.user
        return kwargs



class UserRegistrationMixin(object):
    model = SubjectRegistration

    def get_queryset(self):
        return super(UserRegistrationMixin, self).get_queryset().filter(user=self.request.user)

    def get_template_names(self):
        return [self.template_name] if self.template_name else [
            'leprikon/{}_registration{}.html'.format(self.object.subject.subject_type.slug,
                                                     self.template_name_suffix),
            'leprikon/{}_registration{}.html'.format(self.object.subject.subject_type.subject_type,
                                                     self.template_name_suffix),
            'leprikon/subject_registration{}.html'.format(self.template_name_suffix),
        ]



class SubjectRegistrationPdfView(UserRegistrationMixin, DetailView):

    def get(self, request, *args, **kwargs):
        # get registration
        registration = self.get_object()

        # create PDF response object
        response = HttpResponse(content_type='application/pdf')
        response['Content-Disposition'] = 'attachment; filename="{}"'.format(registration.pdf_filename)

        # create basic pdf registration from rml template
        return registration.write_pdf(response)


class SubjectRegistrationCancelView(UserRegistrationMixin, ConfirmUpdateView):
    template_name_suffix = '_cancel'
    title = _('Registration cancellation request')

    def get_question(self):
        return _('Are you sure You want to cancel the registration "{}"?').format(self.object)

    def get_message(self):
        return _('The cancellation request for {} has been saved.').format(self.object)

    def confirmed(self):
        self.object.cancel_request = True
        self.object.save()


class UserPaymentMixin(object):
    model = SubjectPayment

    def get_queryset(self):
        return super(UserPaymentMixin, self).get_queryset().filter(
            registration__user=self.request.user
        ).order_by('-created')


class SubjectPaymentsListView(UserPaymentMixin, ListView):
    template_name = 'leprikon/payments.html'
    paginate_by = 20

    def get_title(self):
        return _('Payments')


class SubjectPaymentPdfView(UserPaymentMixin, DetailView):
    def get(self, request, *args, **kwargs):
        # get payment
        payment = self.get_object()

        # create PDF response object
        response = HttpResponse(content_type='application/pdf')
        response['Content-Disposition'] = 'attachment; filename="{}"'.format(payment.pdf_filename)

        # create basic pdf payment from rml template
        return payment.write_pdf(response)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-04-23 10:23
from __future__ import unicode_literals

from django.db import migrations, models


def migrate_parents(apps, schema_editor):
    from datetime import date, timedelta

    ClubRegistration    = apps.get_model('leprikon', 'ClubRegistration')
    EventRegistration   = apps.get_model('leprikon', 'EventRegistration')

    for regs in ClubRegistration.objects.all(), EventRegistration.objects.all():
        for reg in regs:
            reg.parents = reg.participant.parents.all()


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0002_messages'),
    ]

    operations = [
        migrations.AddField(
            model_name='clubregistration',
            name='parents',
            field=models.ManyToManyField(blank=True, related_name='club_registrations', to='leprikon.Parent', verbose_name='parents'),
        ),
        migrations.AddField(
            model_name='eventregistration',
            name='parents',
            field=models.ManyToManyField(blank=True, related_name='event_registrations', to='leprikon.Parent', verbose_name='parents'),
        ),
        migrations.RunPython(migrate_parents),
        migrations.RemoveField(
            model_name='participant',
            name='parents',
        ),
    ]



# -*- coding: utf-8 -*-

"""leprikon.site URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/1.9/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  url(r'^$', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  url(r'^$', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.conf.urls import url, include
    2. Add a URL to urlpatterns:  url(r'^blog/', include('blog.urls'))
"""

from django.conf import settings
from django.conf.urls import include, url
from django.conf.urls.static import static
from django.contrib import admin

urlpatterns = [
    url(r'^admin/', include(admin.site.urls)),
    url(r'^social/', include('social_django.urls')),
    url(r'^verified-email-field/', include('verified_email_field.urls')),
    url(r'^', include('cms.urls')),
    # this won't work for displaying pages,
    # but allows reverse resolving before leprikon apphook is attached to any page
    url(r'^leprikon', include('leprikon.urls')),
]

if settings.DEBUG:
    urlpatterns += static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)



"""
WSGI config for leprikon.site project.

It exposes the WSGI callable as a module-level variable named ``application``.

For more information on this file, see
https://docs.djangoproject.com/en/1.9/howto/deployment/wsgi/
"""

import os

from django.core.wsgi import get_wsgi_application
from raven.contrib.django.raven_compat.middleware.wsgi import Sentry

os.environ.setdefault(
    "DJANGO_SETTINGS_MODULE",
    "{}.settings".format(os.environ.get('SITE_MODULE', 'leprikon.site')),
)

application = Sentry(get_wsgi_application())



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from ..forms.leaders import LeaderFilterForm
from ..models.roles import Leader
from .generic import FilteredListView


class LeaderListView(FilteredListView):
    model               = Leader
    form_class          = LeaderFilterForm
    preview_template    = 'leprikon/leader_preview.html'
    template_name       = 'leprikon/leader_list.html'
    message_empty       = _('No leaders matching given query.')
    paginate_by         = 10

    def get_title(self):
        return _('Leaders in school year {}').format(self.request.school_year)

    def get_form(self):
        return self.form_class(
            school_year = self.request.school_year,
            data = self.request.GET,
        )

    def get_queryset(self):
        return self.get_form().get_queryset()



from __future__ import unicode_literals

import colorsys
from collections import OrderedDict
from io import BytesIO
from json import loads

import trml2pdf
from cms.models.fields import PageField
from django.core.exceptions import ValidationError
from django.core.mail import EmailMultiAlternatives
from django.core.urlresolvers import reverse_lazy as reverse
from django.db import models
from django.template.loader import select_template
from django.utils import timezone
from django.utils.encoding import (
    force_text, python_2_unicode_compatible, smart_text,
)
from django.utils.functional import cached_property
from django.utils.text import slugify
from django.utils.translation import ugettext_lazy as _
from django_countries.fields import CountryField
from djangocms_text_ckeditor.fields import HTMLField
from filer.fields.file import FilerFileField
from filer.fields.image import FilerImageField
from PyPDF2 import PdfFileReader, PdfFileWriter

from ..conf import settings
from ..utils import comma_separated, currency, get_birth_date
from .agegroup import AgeGroup
from .fields import BirthNumberField, ColorField, PostalCodeField, PriceField
from .insurance import Insurance
from .leprikonsite import LeprikonSite
from .place import Place
from .printsetup import PrintSetup
from .question import Question
from .roles import Leader, Manager
from .school import School
from .schoolyear import SchoolYear
from .utils import generate_variable_symbol


@python_2_unicode_compatible
class SubjectType(models.Model):
    COURSE = 'course'
    EVENT = 'event'
    subject_type_labels = OrderedDict([
        (COURSE, _('course')),
        (EVENT, _('event')),
    ])
    subject_type_type_labels = OrderedDict([
        (COURSE, _('course type')),
        (EVENT, _('event type')),
    ])
    subject_type    = models.CharField(_('subjects'), max_length=10, choices=subject_type_labels.items())
    name            = models.CharField(_('name'), max_length=150)
    name_genitiv    = models.CharField(_('name (genitiv)'), max_length=150, blank=True)
    name_akuzativ   = models.CharField(_('name (akuzativ)'), max_length=150, blank=True)
    plural          = models.CharField(_('name (plural)'), max_length=150)
    slug            = models.SlugField()
    order           = models.IntegerField(_('order'), blank=True, default=0)
    questions       = models.ManyToManyField(
        Question, verbose_name=_('additional questions'), blank=True, related_name='+',
        help_text=_('Add additional questions to be asked in the registration form.'),
    )
    agreement       = models.TextField(_('registration agreement'), blank=True, null=True)
    reg_print_setup = models.ForeignKey(PrintSetup, on_delete=models.SET_NULL, related_name='+',
                                        verbose_name=_('registration print setup'), blank=True, null=True)
    bill_print_setup = models.ForeignKey(PrintSetup, on_delete=models.SET_NULL, related_name='+',
                                         verbose_name=_('payment print setup'), blank=True, null=True)

    class Meta:
        app_label           = 'leprikon'
        ordering            = ('order',)
        verbose_name        = _('subject type')
        verbose_name_plural = _('subject types')

    def __str__(self):
        return self.name

    def save(self, *args, **kwargs):
        if self.name_genitiv is None:
            self.name_genitiv = self.name
        if self.name_akuzativ is None:
            self.name_akuzativ = self.name
        super(SubjectType, self).save(*args, **kwargs)

    @cached_property
    def all_questions(self):
        return list(self.questions.all())

    @cached_property
    def all_attachments(self):
        return list(self.attachments.all())



@python_2_unicode_compatible
class SubjectTypeAttachment(models.Model):
    subject_type    = models.ForeignKey(SubjectType, verbose_name=_('subject type'), related_name='attachments')
    file            = FilerFileField(related_name='+')
    order           = models.IntegerField(_('order'), blank=True, default=0)

    class Meta:
        app_label           = 'leprikon'
        ordering            = ('order',)
        verbose_name        = _('attachment')
        verbose_name_plural = _('attachments')

    def __str__(self):
        return force_text(self.file)



@python_2_unicode_compatible
class SubjectGroup(models.Model):
    name    = models.CharField(_('name'), max_length=150)
    plural  = models.CharField(_('plural'), max_length=150)
    color   = ColorField(_('color'))
    order   = models.IntegerField(_('order'), blank=True, default=0)
    subject_types = models.ManyToManyField(SubjectType, verbose_name=_('subject type'), related_name='groups')

    class Meta:
        app_label           = 'leprikon'
        ordering            = ('order',)
        verbose_name        = _('subject group')
        verbose_name_plural = _('subject groups')

    def __str__(self):
        return self.name

    @cached_property
    def font_color(self):
        (h, s, v) = colorsys.rgb_to_hsv(
            int(self.color[1:3], 16) / 255.0,
            int(self.color[3:5], 16) / 255.0,
            int(self.color[5:6], 16) / 255.0,
        )
        if v > .5:
            v = 0
        else:
            v = 1
        if s > .5:
            s = 0
        else:
            s = 1
        (r, g, b) = colorsys.hsv_to_rgb(h, s, v)
        return '#{:02x}{:02x}{:02x}'.format(
            int(r * 255),
            int(g * 255),
            int(b * 255),
        )



@python_2_unicode_compatible
class Subject(models.Model):
    school_year = models.ForeignKey(SchoolYear, verbose_name=_('school year'), related_name='subjects', editable=False)
    subject_type = models.ForeignKey(SubjectType, verbose_name=_('subject type'),
                                     related_name='subjects', on_delete=models.PROTECT)
    code        = models.PositiveSmallIntegerField(_('accounting code'), blank=True, default=0)
    name        = models.CharField(_('name'), max_length=150)
    description = HTMLField(_('description'), blank=True, default='')
    managers    = models.ManyToManyField(Manager, verbose_name=_('managers'), related_name='subjects', blank=True)
    groups      = models.ManyToManyField(SubjectGroup, verbose_name=_('groups'), related_name='subjects', blank=True)
    place       = models.ForeignKey(Place, verbose_name=_('place'), blank=True, null=True,
                                    related_name='subjects', on_delete=models.SET_NULL)
    age_groups  = models.ManyToManyField(AgeGroup, verbose_name=_('age groups'), related_name='subjects', blank=True)
    leaders     = models.ManyToManyField(Leader, verbose_name=_('leaders'), related_name='subjects', blank=True)
    price       = PriceField(_('price'), blank=True, null=True)
    public      = models.BooleanField(_('public'), default=False)
    reg_from    = models.DateTimeField(_('registration active from'), blank=True, null=True)
    reg_to      = models.DateTimeField(_('registration active to'), blank=True, null=True)
    photo       = FilerImageField(verbose_name=_('photo'), blank=True, null=True,
                                  related_name='+', on_delete=models.SET_NULL)
    page        = PageField(verbose_name=_('page'), blank=True, null=True,
                            related_name='+', on_delete=models.SET_NULL)
    min_count   = models.IntegerField(_('minimal count'), blank=True, null=True)
    max_count   = models.IntegerField(_('maximal count'), blank=True, null=True)
    risks       = HTMLField(_('risks'), blank=True)
    plan        = HTMLField(_('plan'), blank=True)
    evaluation  = HTMLField(_('evaluation'), blank=True)
    note        = models.CharField(_('note'), max_length=300, blank=True, default='')
    questions   = models.ManyToManyField(Question, verbose_name=_('additional questions'),
                                         related_name='+', blank=True,
                                         help_text=_('Add additional questions to be asked in the registration form.'))
    agreement       = models.TextField(_('registration agreement'), blank=True, null=True)
    reg_print_setup = models.ForeignKey(PrintSetup, on_delete=models.SET_NULL, related_name='+',
                                        verbose_name=_('registration print setup'), blank=True, null=True)
    bill_print_setup = models.ForeignKey(PrintSetup, on_delete=models.SET_NULL, related_name='+',
                                         verbose_name=_('payment print setup'), blank=True, null=True)


    class Meta:
        app_label           = 'leprikon'
        ordering            = ('name',)
        verbose_name        = _('subject')
        verbose_name_plural = _('subjects')

    def __str__(self):
        return '{} {}'.format(self.school_year, self.name)

    @cached_property
    def all_variants(self):
        return list(self.variants.all())

    @cached_property
    def all_managers(self):
        return list(self.managers.all())

    @cached_property
    def all_groups(self):
        return list(self.groups.all())

    @cached_property
    def all_age_groups(self):
        return list(self.age_groups.all())

    @cached_property
    def all_leaders(self):
        return list(self.leaders.all())

    @cached_property
    def all_questions(self):
        return set(self.subject_type.all_questions + list(self.questions.all()))

    @cached_property
    def all_attachments(self):
        return list(self.attachments.all())

    @cached_property
    def all_registrations(self):
        return list(self.registrations.all())

    @property
    def registration_allowed(self):
        now = timezone.now()
        return (
            self.price is not None and
            self.reg_from is not None and
            self.reg_from <= now and
            (self.reg_to is None or self.reg_to > now)
        )

    @property
    def full(self):
        return self.max_count and self.approved_registrations_count >= self.max_count

    def get_absolute_url(self):
        return reverse('leprikon:subject_detail', args=(self.subject_type.slug, self.id))

    def get_registration_url(self):
        return reverse('leprikon:subject_registration_form',
                       kwargs={'subject_type': self.subject_type.slug, 'pk': self.id})

    def get_edit_url(self):
        return reverse('admin:leprikon_{}_change'.format(self._meta.model_name), args=(self.id,))

    def get_groups_list(self):
        return comma_separated(self.all_groups)
    get_groups_list.short_description = _('groups list')

    def get_leaders_list(self):
        return comma_separated(self.all_leaders)
    get_leaders_list.short_description = _('leaders list')

    @property
    def active_registrations(self):
        return self.registrations.filter(canceled=None)

    @property
    def approved_registrations(self):
        return self.active_registrations.exclude(approved=None)

    @cached_property
    def approved_registrations_count(self):
        return self.approved_registrations.count()

    @cached_property
    def all_approved_registrations(self):
        return list(self.approved_registrations.all())

    @property
    def unapproved_registrations(self):
        return self.active_registrations.filter(approved=None)

    @cached_property
    def unapproved_registrations_count(self):
        return self.unapproved_registrations.count()

    @cached_property
    def all_unapproved_registrations(self):
        return list(self.unapproved_registrations.all())

    @cached_property
    def inactive_registrations_count(self):
        return self.inactive_registrations.count()

    @cached_property
    def all_inactive_registrations(self):
        return list(self.inactive_registrations.all())

    def get_managers_list(self):
        return comma_separated(self.all_managers)
    get_managers_list.short_description = _('managers list')

    def get_agreement(self):
        return (
            self.agreement or
            self.subject_type.agreement or
            LeprikonSite.objects.get_current().agreement or
            _('I confirm that I have read, understood and agree with the Terms and Conditions. '
              'The terms and conditions are available on the web.')
        )



@python_2_unicode_compatible
class SubjectVariant(models.Model):
    subject     = models.ForeignKey(Subject, verbose_name=_('subject'), related_name='variants')
    name        = models.CharField(_('variant name'), max_length=150)
    description = HTMLField(_('variant description'), blank=True, default='')
    price       = PriceField(_('price'), blank=True, null=True)
    order       = models.IntegerField(_('order'), blank=True, default=0)

    class Meta:
        app_label           = 'leprikon'
        ordering            = ('order',)
        verbose_name        = _('variant')
        verbose_name_plural = _('variants')

    def __str__(self):
        return self.name



@python_2_unicode_compatible
class SubjectAttachment(models.Model):
    subject = models.ForeignKey(Subject, verbose_name=_('subject'), related_name='attachments')
    file    = FilerFileField(related_name='+')
    order   = models.IntegerField(_('order'), blank=True, default=0)

    class Meta:
        app_label           = 'leprikon'
        ordering            = ('order',)
        verbose_name        = _('attachment')
        verbose_name_plural = _('attachments')

    def __str__(self):
        return force_text(self.file)



class PdfExportMixin(object):
    all_attachments = []

    def get_absolute_url(self):
        return reverse('leprikon:{}_pdf'.format(self.pdf_export), kwargs={'pk': self.pk, 'slug': self.slug})

    def send_mail(self):
        template_txt = select_template([
            'leprikon/{}-mail/{}.txt'.format(self.pdf_export, self.subject.subject_type.slug),
            'leprikon/{}-mail/{}.txt'.format(self.pdf_export, self.subject.subject_type.subject_type),
            'leprikon/{}-mail/subject.txt'.format(self.pdf_export),
        ])
        template_html = select_template([
            'leprikon/{}-mail/{}.html'.format(self.pdf_export, self.subject.subject_type.slug),
            'leprikon/{}-mail/{}.html'.format(self.pdf_export, self.subject.subject_type.subject_type),
            'leprikon/{}-mail/subject.html'.format(self.pdf_export),
        ])
        context = {
            'object': self,
            'site': LeprikonSite.objects.get_current(),
        }
        content_txt = template_txt.render(context)
        content_html = template_html.render(context)
        msg = EmailMultiAlternatives(
            subject = self.mail_subject,
            body        = content_txt,
            from_email  = settings.SERVER_EMAIL,
            to          = self.all_recipients,
            headers     = {'X-Mailer': 'Leprikon (http://leprikon.cz/)'},
        )
        msg.attach_alternative(content_html, 'text/html')
        msg.attach(self.pdf_filename, self.get_pdf(), 'application/pdf')
        for attachment in self.all_attachments:
            msg.attach_file(attachment.file.file.path)
        msg.send()

    @cached_property
    def pdf_filename(self):
        return self.slug + '.pdf'

    def get_pdf(self):
        output = BytesIO()
        self.write_pdf(output)
        output.seek(0)
        return output.read()

    def write_pdf(self, output):
        # get plain pdf from rml
        template = select_template([
            'leprikon/{}/{}.rml'.format(self.pdf_export, self.subject.subject_type.slug),
            'leprikon/{}/{}.rml'.format(self.pdf_export, self.subject.subject_type.subject_type),
            'leprikon/{}/subject.rml'.format(self.pdf_export),
        ])
        rml_content = template.render({
            'object': self,
            'site': LeprikonSite.objects.get_current(),
        })
        pdf_content = trml2pdf.parseString(rml_content.encode('utf-8'))

        # merge with background
        if self.print_setup.background:
            template_pdf = PdfFileReader(self.print_setup.background.file)
            registration_pdf = PdfFileReader(BytesIO(pdf_content))
            writer = PdfFileWriter()
            # merge pages from both template and registration
            for i in range(registration_pdf.getNumPages()):
                if i < template_pdf.getNumPages():
                    page = template_pdf.getPage(i)
                    page.mergePage(registration_pdf.getPage(i))
                else:
                    page = registration_pdf.getPage(i)
                writer.addPage(page)
            # write result to output
            writer.write(output)
        else:
            # write basic pdf registration to response
            output.write(pdf_content)
        return output


@python_2_unicode_compatible
class SubjectRegistration(PdfExportMixin, models.Model):
    slug            = models.SlugField(editable=False)
    created         = models.DateTimeField(_('time of registration'), editable=False, auto_now_add=True)
    user            = models.ForeignKey(settings.AUTH_USER_MODEL, verbose_name=_('user'),
                                        related_name='leprikon_registrations', on_delete=models.PROTECT)
    subject         = models.ForeignKey(Subject, verbose_name=_('subject'),
                                        related_name='registrations', on_delete=models.PROTECT)
    subject_variant = models.ForeignKey(SubjectVariant, null=True, verbose_name=_('variant'),
                                        related_name='registrations', on_delete=models.PROTECT)
    price           = PriceField(_('price'), editable=False)
    answers         = models.TextField(_('additional answers'), blank=True, default='{}', editable=False)

    approved        = models.DateTimeField(_('time of approval'), editable=False, null=True)
    payment_requested = models.DateTimeField(_('payment request time'), editable=False, null=True)
    canceled        = models.DateTimeField(_('time of cancellation'), editable=False, null=True)
    cancel_request  = models.BooleanField(_('cancel request'), default=False)

    MALE = 'm'
    FEMALE = 'f'
    participant_gender      = models.CharField(_('gender'), max_length=1, editable=False,
                                               choices=((MALE, _('male')), (FEMALE, _('female'))))
    participant_first_name  = models.CharField(_('first name'),   max_length=30)
    participant_last_name   = models.CharField(_('last name'),    max_length=30)
    participant_birth_num   = BirthNumberField(_('birth number'))
    participant_age_group   = models.ForeignKey(AgeGroup, verbose_name=_('age group'),
                                                related_name='+', on_delete=models.PROTECT)
    participant_street      = models.CharField(_('street'),       max_length=150)
    participant_city        = models.CharField(_('city'),         max_length=150)
    participant_postal_code = PostalCodeField(_('postal code'))
    participant_citizenship = CountryField(_('citizenship'))
    participant_insurance   = models.ForeignKey(Insurance, verbose_name=_('insurance'), null=True,
                                                related_name='+', on_delete=models.PROTECT)
    participant_phone       = models.CharField(_('phone'),        max_length=30, blank=True, default='')
    participant_email       = models.EmailField(_('email address'),              blank=True, default='')

    participant_school          = models.ForeignKey(School, verbose_name=_('school'), blank=True, null=True,
                                                    related_name='+', on_delete=models.PROTECT)
    participant_school_other    = models.CharField(_('other school'), max_length=150, blank=True, default='')
    participant_school_class    = models.CharField(_('class'),        max_length=30,  blank=True, default='')
    participant_health          = models.TextField(_('health'), blank=True, default='')

    has_parent1         = models.BooleanField(_('first parent'), default=False)
    parent1_first_name  = models.CharField(_('first name'),   max_length=30,  blank=True, null=True)
    parent1_last_name   = models.CharField(_('last name'),    max_length=30,  blank=True, null=True)
    parent1_street      = models.CharField(_('street'),       max_length=150, blank=True, null=True)
    parent1_city        = models.CharField(_('city'),         max_length=150, blank=True, null=True)
    parent1_postal_code = PostalCodeField(_('postal code'),                   blank=True, null=True)
    parent1_phone       = models.CharField(_('phone'),        max_length=30,  blank=True, null=True)
    parent1_email       = models.EmailField(_('email address'),               blank=True, null=True)

    has_parent2         = models.BooleanField(_('second parent'), default=False)
    parent2_first_name  = models.CharField(_('first name'),   max_length=30,  blank=True, null=True)
    parent2_last_name   = models.CharField(_('last name'),    max_length=30,  blank=True, null=True)
    parent2_street      = models.CharField(_('street'),       max_length=150, blank=True, null=True)
    parent2_city        = models.CharField(_('city'),         max_length=150, blank=True, null=True)
    parent2_postal_code = PostalCodeField(_('postal code'),                   blank=True, null=True)
    parent2_phone       = models.CharField(_('phone'),        max_length=30,  blank=True, null=True)
    parent2_email       = models.EmailField(_('email address'),               blank=True, null=True)

    variable_symbol     = models.IntegerField(_('variable symbol'), db_index=True, editable=False, null=True)

    class Meta:
        app_label           = 'leprikon'
        verbose_name        = _('registration')
        verbose_name_plural = _('registrations')

    def __str__(self):
        return _('{participant} ({birth_num})').format(
            participant = self.participant,
            birth_num   = self.participant_birth_num,
        )

    def validate_unique(self, exclude=None):
        super(SubjectRegistration, self).validate_unique(None)
        qs = SubjectRegistration.objects.filter(
            subject=self.subject,
            participant_birth_num=self.participant_birth_num,
            canceled=None)
        if self.pk:
            qs = qs.exclude(pk=self.pk)
        if qs.exists():
            raise ValidationError(
                message={'participant_birth_num': _('Participant with this birth number has already been registered.')},
            )

    def get_answers(self):
        return loads(self.answers)

    def get_questions_and_answers(self):
        answers = self.get_answers()
        for q in self.subject.all_questions:
            yield {
                'question': q.question,
                'answer': answers.get(q.name, ''),
            }

    @cached_property
    def subjectregistration(self):
        if self.subject.subject_type.subject_type == self.subject.subject_type.COURSE:
            return self.courseregistration
        else:
            return self.eventregistration

    @cached_property
    def participant(self):
        return self.Person(self, 'participant')
    participant.short_description = _('participant')

    @cached_property
    def parents(self):
        return list(p for p in [self.parent1, self.parent2] if p is not None)

    @cached_property
    def parent1(self):
        if self.has_parent1:
            return self.Person(self, 'parent1')
        else:
            return None
    parent1.short_description = _('first parent')

    @cached_property
    def parent2(self):
        if self.has_parent2:
            return self.Person(self, 'parent2')
        else:
            return None
    parent2.short_description = _('second parent')

    @cached_property
    def all_attachments(self):
        return self.subject.all_attachments + self.subject.subject_type.all_attachments

    @cached_property
    def all_recipients(self):
        recipients = set()
        if self.user.email:
            recipients.add(self.user.email)
        for parent in self.parents:
            if parent.email:
                recipients.add(parent.email)
        return recipients

    @cached_property
    def all_discounts(self):
        return list(self.discounts.all())

    @cached_property
    def all_payments(self):
        return list(self.payments.all())

    def get_discounts(self, d):
        if d:
            return list(filter(lambda p: p.created.date() <= d, self.all_discounts))
        else:
            return self.all_discounts

    def get_payments(self, d):
        if d:
            return list(filter(lambda p: p.created.date() <= d, self.all_payments))
        else:
            return self.all_payments

    def get_discounted(self, d=None):
        return sum(p.amount for p in self.get_discounts(d))

    def get_paid(self, d=None):
        return sum(p.amount for p in self.get_payments(d))

    def get_current_receivable(self):
        return self.subjectregistration.get_current_receivable()

    def approve(self):
        if self.approved is None:
            self.approved = timezone.now()
            self.save()

    def request_payment(self):
        if self.payment_requested is None:
            self.payment_requested = timezone.now()
            self.save()
            try:
                self.send_payment_request()
            except:
                import traceback
                from raven.contrib.django.models import client
                traceback.print_exc()
                client.captureException()

    def cancel(self):
        if self.canceled is None:
            self.canceled = timezone.now()
            self.save()

    def save(self, *args, **kwargs):
        self.participant_gender = self.participant_birth_num[2:4] > '50' and self.FEMALE or self.MALE
        if not self.slug:
            self.slug = slugify('{}-{}'.format(self.participant.full_name, self.subject.name))[:50]
        if self.canceled:
            self.cancel_request = False
        if not self.has_parent1:
            if self.has_parent2:
                self.parent1_first_name  = self.parent2_first_name
                self.parent1_last_name   = self.parent2_last_name
                self.parent1_street      = self.parent2_street
                self.parent1_city        = self.parent2_city
                self.parent1_postal_code = self.parent2_postal_code
                self.parent1_phone       = self.parent2_phone
                self.parent1_email       = self.parent2_email
                self.has_parent1 = True
                self.has_parent2 = False
            else:
                self.parent1_first_name  = None
                self.parent1_last_name   = None
                self.parent1_street      = None
                self.parent1_city        = None
                self.parent1_postal_code = None
                self.parent1_phone       = None
                self.parent1_email       = None
        if not self.has_parent2:
            self.parent2_first_name  = None
            self.parent2_last_name   = None
            self.parent2_street      = None
            self.parent2_city        = None
            self.parent2_postal_code = None
            self.parent2_phone       = None
            self.parent2_email       = None
        super(SubjectRegistration, self).save(*args, **kwargs)
        if not self.variable_symbol:
            self.variable_symbol = generate_variable_symbol(self)
            super(SubjectRegistration, self).save(*args, **kwargs)

    @cached_property
    def agreement(self):
        return self.subject.get_agreement()

    pdf_export = 'registration'

    @cached_property
    def print_setup(self):
        return (
            self.subject.reg_print_setup or
            self.subject.subject_type.reg_print_setup or
            LeprikonSite.objects.get_current().reg_print_setup or
            PrintSetup()
        )

    @cached_property
    def mail_subject(self):
        return _('Registration for {subject_type} accepted').format(
            subject_type=self.subject.subject_type.name_akuzativ
        )

    @cached_property
    def payment_request_subject(self):
        return _('Registration for {subject_type} - payment information').format(
            subject_type=self.subject.subject_type.name_akuzativ
        )

    def send_payment_request(self):
        template_txt = select_template([
            'leprikon/payment-request-mail/{}.txt'.format(self.subject.subject_type.slug),
            'leprikon/payment-request-mail/{}.txt'.format(self.subject.subject_type.subject_type),
            'leprikon/payment-request-mail/subject.txt',
        ])
        template_html = select_template([
            'leprikon/payment-request-mail/{}.html'.format(self.subject.subject_type.slug),
            'leprikon/payment-request-mail/{}.html'.format(self.subject.subject_type.subject_type),
            'leprikon/payment-request-mail/subject.html',
        ])
        context = {
            'object': self,
            'site': LeprikonSite.objects.get_current(),
        }
        content_txt = template_txt.render(context)
        content_html = template_html.render(context)
        msg = EmailMultiAlternatives(
            subject = self.payment_request_subject,
            body        = content_txt,
            from_email  = settings.SERVER_EMAIL,
            to          = self.all_recipients,
            headers     = {'X-Mailer': 'Leprikon (http://leprikon.cz/)'},
        )
        msg.attach_alternative(content_html, 'text/html')
        msg.send()

    @python_2_unicode_compatible
    class Person:
        def __init__(self, registration, role):
            self._registration = registration
            self._role = role

        def __getattr__(self, attr):
            return getattr(self._registration, '{}_{}'.format(self._role, attr))

        def __str__(self):
            return self.full_name

        @cached_property
        def full_name(self):
            return '{} {}'.format(self.first_name, self.last_name)

        @cached_property
        def address(self):
            return '{}, {}, {}'.format(self.street, self.city, self.postal_code)

        @cached_property
        def contact(self):
            if self.email and self.phone:
                return '{}, {}'.format(self.phone, self.email)
            else:
                return self.email or self.phone or ''

        # this is participant specific
        @cached_property
        def school_name(self):
            return self.school and smart_text(self.school) or self.school_other

        @cached_property
        def school_and_class(self):
            if self.school_name and self.school_class:
                return '{}, {}'.format(self.school_name, self.school_class)
            else:
                return self.school_name or self.school_class or ''

        @cached_property
        def birth_date(self):
            return get_birth_date(self.birth_num)



@python_2_unicode_compatible
class SubjectDiscount(models.Model):
    created     = models.DateTimeField(_('time of discount'), editable=False, auto_now_add=True)
    amount      = PriceField(_('discount'), default=0)
    explanation = models.CharField(_('discount explanation'), max_length=250, blank=True, default='')

    class Meta:
        abstract            = True
        app_label           = 'leprikon'
        verbose_name        = _('discount')
        verbose_name_plural = _('discounts')
        ordering            = ('created',)

    def __str__(self):
        return currency(self.amount)

    def save(self, *args, **kwargs):
        self.registration.approve()
        super(SubjectDiscount, self).save(*args, **kwargs)



@python_2_unicode_compatible
class SubjectPayment(PdfExportMixin, models.Model):
    PAYMENT_CASH = 'PAYMENT_CASH'
    PAYMENT_BANK = 'PAYMENT_BANK'
    PAYMENT_ONLINE = 'PAYMENT_ONLINE'
    PAYMENT_TRANSFER = 'PAYMENT_TRANSFER'
    RETURN_CASH = 'RETURN_CASH'
    RETURN_BANK = 'RETURN_BANK'
    RETURN_TRANSFER = 'RETURN_TRANSFER'
    payment_type_labels = OrderedDict([
        (PAYMENT_CASH, _('payment - cash')),
        (PAYMENT_BANK, _('payment - bank')),
        (PAYMENT_ONLINE, _('payment - online')),
        (PAYMENT_TRANSFER, _('payment - transfer from return')),
        (RETURN_CASH, _('return - cash')),
        (RETURN_BANK, _('return - bank')),
        (RETURN_TRANSFER, _('return - transfer to payment')),
    ])
    registration    = models.ForeignKey(SubjectRegistration, verbose_name=_('registration'),
                                        related_name='payments', on_delete=models.PROTECT)
    created         = models.DateTimeField(_('payment time'), editable=False, auto_now_add=True)
    payment_type    = models.CharField(_('payment type'), max_length=30, choices=payment_type_labels.items())
    amount          = PriceField(_('amount'), help_text=_('positive value for payment, negative value for return'))
    note            = models.CharField(_('note'), max_length=300, blank=True, default='')
    related_payment = models.ForeignKey('self', verbose_name=_('related payment'), blank=True, null=True,
                                        related_name='related_payments', on_delete=models.PROTECT)
    received_by     = models.ForeignKey(settings.AUTH_USER_MODEL, null=True, verbose_name=_('received by'),
                                        related_name='+', on_delete=models.PROTECT)

    payments = frozenset({PAYMENT_CASH, PAYMENT_BANK, PAYMENT_ONLINE, PAYMENT_TRANSFER})
    returns = frozenset({RETURN_CASH, RETURN_BANK, RETURN_TRANSFER})

    class Meta:
        app_label           = 'leprikon'
        verbose_name        = _('payment')
        verbose_name_plural = _('payments')
        ordering            = ('created',)

    def __str__(self):
        return '{registration}, {payment_type} {amount}'.format(
            registration    = self.registration,
            payment_type    = self.payment_type_label,
            amount          = currency(abs(self.amount)),
        )

    @cached_property
    def payment_type_label(self):
        return self.payment_type_labels.get(self.payment_type, '-')
    payment_type_label.short_description = _('payment type')

    pdf_export = 'payment'

    @cached_property
    def slug(self):
        return '{}-{}'.format(self.registration.slug, self.id)

    @cached_property
    def print_setup(self):
        return (
            self.registration.subject.bill_print_setup or
            self.registration.subject.subject_type.bill_print_setup or
            LeprikonSite.objects.get_current().bill_print_setup or
            PrintSetup()
        )

    @cached_property
    def subject(self):
        return self.registration.subject

    @cached_property
    def mail_subject(self):
        return _('Registration for {subject_type} accepted').format(
            subject_type=self.subject.subject_type.name_akuzativ
        )

    def validate(self):
        if self.amount == 0:
            raise ValidationError({
                'amount': [_('Amount can not be zero.')]
            })
        if self.payment_type:
            if self.amount > 0 and self.payment_type not in self.payments:
                raise ValidationError({
                    'payment_type': [_('Select one of the payments or use negative amount.')],
                })
            if self.amount < 0 and self.payment_type not in self.returns:
                raise ValidationError({
                    'payment_type': [_('Select one of the returns or use positive amount.')],
                })

    def clean_fields(self, exclude=None):
        super(SubjectPayment, self).clean_fields(exclude)
        self.validate()

    def save(self, *args, **kwargs):
        self.validate()
        self.registration.approve()
        super(SubjectPayment, self).save(*args, **kwargs)



from __future__ import unicode_literals

from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _


@python_2_unicode_compatible
class Insurance(models.Model):
    code    = models.CharField(_('code'), max_length=10, unique=True)
    name    = models.CharField(_('name'), max_length=150)

    class Meta:
        app_label           = 'leprikon'
        ordering            = ('code',)
        verbose_name        = _('insurance company')
        verbose_name_plural = _('insurance companies')

    def __str__(self):
        return '{} - {}'.format(self.code, self.name)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-05-29 12:14
from __future__ import unicode_literals

import cms.models.fields
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0005_messageattachment'),
    ]

    operations = [
        migrations.AlterField(
            model_name='club',
            name='page',
            field=cms.models.fields.PageField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='cms.Page', verbose_name='page'),
        ),
        migrations.AlterField(
            model_name='event',
            name='page',
            field=cms.models.fields.PageField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='cms.Page', verbose_name='page'),
        ),
        migrations.AlterField(
            model_name='leader',
            name='page',
            field=cms.models.fields.PageField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='cms.Page', verbose_name='page'),
        ),
    ]



from __future__ import unicode_literals

from collections import namedtuple
from datetime import date, datetime

from cms.models import CMSPlugin
from django.db import models
from django.utils.formats import date_format, time_format
from django.utils.functional import cached_property
from django.utils.translation import ugettext_lazy as _

from ..conf import settings
from .agegroup import AgeGroup
from .roles import Leader, Manager
from .schoolyear import SchoolYear
from .subjects import (
    Subject, SubjectDiscount, SubjectGroup, SubjectRegistration, SubjectType,
)
from .utils import PaymentStatus


class Event(Subject):
    start_date  = models.DateField(_('start date'))
    end_date    = models.DateField(_('end date'))
    start_time  = models.TimeField(_('start time'), blank=True, null=True)
    end_time    = models.TimeField(_('end time'), blank=True, null=True)

    class Meta:
        app_label           = 'leprikon'
        ordering            = ('-start_date', 'start_time')
        verbose_name        = _('event')
        verbose_name_plural = _('events')

    @property
    def inactive_registrations(self):
        return self.registrations.filter(canceled__isnull=False)

    def get_approved_registrations(self, d):
        return self.registrations.filter(created__lte=d, approved__lte=d).exclude(canceled__lt=d)

    def get_times_list(self):
        return '{start}{separator}{end}'.format(
            start = (date_format(datetime.combine(self.start_date, self.start_time), 'SHORT_DATETIME_FORMAT')
                     if self.start_time else date_format(self.start_date, 'SHORT_DATE_FORMAT')),
            separator = ' - ' if self.start_date != self.end_date or self.end_time is not None else '',
            end = ((time_format(self.end_time, 'TIME_FORMAT') if self.end_time else '')
                   if self.start_date == self.end_date
                   else (date_format(datetime.combine(self.end_date, self.end_time), 'SHORT_DATETIME_FORMAT')
                         if self.end_time else date_format(self.end_date, 'SHORT_DATE_FORMAT'))),
        )
    get_times_list.short_description = _('times')

    def copy_to_school_year(old, school_year):
        new = Event.objects.get(id=old.id)
        new.id, new.pk = None, None
        new.school_year = school_year
        new.public      = False
        new.evaluation  = ''
        new.note        = ''
        year_offset = school_year.year - old.school_year.year
        if new.reg_from:
            try:
                new.reg_from = datetime(
                    new.reg_from.year + year_offset,
                    new.reg_from.month,
                    new.reg_from.day,
                    new.reg_from.hour,
                    new.reg_from.minute,
                    new.reg_from.second,
                )
            except ValueError:
                # handle leap-year
                new.reg_from = datetime(
                    new.reg_from.year + year_offset,
                    new.reg_from.month,
                    new.reg_from.day - 1,
                    new.reg_from.hour,
                    new.reg_from.minute,
                    new.reg_from.second,
                )
        if new.reg_to:
            try:
                new.reg_to = datetime(
                    new.reg_to.year + year_offset,
                    new.reg_to.month,
                    new.reg_to.day,
                    new.reg_to.hour,
                    new.reg_to.minute,
                    new.reg_to.second,
                )
            except ValueError:
                # handle leap-year
                new.reg_to = datetime(
                    new.reg_to.year + year_offset,
                    new.reg_to.month,
                    new.reg_to.day - 1,
                    new.reg_to.hour,
                    new.reg_to.minute,
                    new.reg_to.second,
                )
        try:
            new.start_date = date(new.start_date.year + year_offset, new.start_date.month, new.start_date.day)
        except ValueError:
            # handle leap-year
            new.start_date = date(new.start_date.year + year_offset, new.start_date.month, new.start_date.day - 1)
        try:
            new.end_date   = date(new.end_date.year   + year_offset, new.end_date.month,   new.end_date.day)
        except ValueError:
            # handle leap-year
            new.end_date   = date(new.end_date.year   + year_offset, new.end_date.month,   new.end_date.day - 1)
        new.save()
        new.managers    = old.managers.all()
        new.groups      = old.groups.all()
        new.age_groups  = old.age_groups.all()
        new.leaders     = old.leaders.all()
        new.questions   = old.questions.all()
        new.attachments = old.attachments.all()
        return new



class EventRegistration(SubjectRegistration):

    class Meta:
        app_label           = 'leprikon'
        verbose_name        = _('event registration')
        verbose_name_plural = _('event registrations')

    @cached_property
    def payment_status(self):
        return self.get_payment_status()

    def get_payment_status(self, d=None):
        return PaymentStatus(
            price       = self.price if self.approved and (d is None or d >= self.approved.date()) else 0,
            discount    = self.get_discounted(d),
            paid        = self.get_paid(d),
        )

    def get_current_receivable(self):
        d = date.today()
        price = self.price
        discount = self.get_discounted(d)
        paid = self.get_paid(d)
        return max(price - discount - paid, 0)



class EventDiscount(SubjectDiscount):
    registration    = models.ForeignKey(EventRegistration, verbose_name=_('registration'),
                                        related_name='discounts', on_delete=models.PROTECT)



class EventPlugin(CMSPlugin):
    event       = models.ForeignKey(Event, verbose_name=_('event'), related_name='+')
    template    = models.CharField(_('template'), max_length=100,
                                   choices=settings.LEPRIKON_EVENT_TEMPLATES,
                                   default=settings.LEPRIKON_EVENT_TEMPLATES[0][0],
                                   help_text=_('The template used to render plugin.'))

    class Meta:
        app_label = 'leprikon'



class EventListPlugin(CMSPlugin):
    school_year = models.ForeignKey(SchoolYear, verbose_name=_('school year'),
                                    related_name='+', blank=True, null=True)
    managers    = models.ManyToManyField(Manager, verbose_name=_('managers'), blank=True, related_name='+',
                                         help_text=_('Keep empty to skip searching by managers.'))
    event_types = models.ManyToManyField(SubjectType, verbose_name=_('event types'), blank=True, related_name='+',
                                         limit_choices_to={'subject_type': SubjectType.EVENT},
                                         help_text=_('Keep empty to skip searching by event types.'))
    age_groups  = models.ManyToManyField(AgeGroup, verbose_name=_('age groups'), blank=True, related_name='+',
                                         help_text=_('Keep empty to skip searching by age groups.'))
    groups      = models.ManyToManyField(SubjectGroup, verbose_name=_('event groups'), blank=True, related_name='+',
                                         help_text=_('Keep empty to skip searching by groups.'))
    leaders     = models.ManyToManyField(Leader, verbose_name=_('leaders'), blank=True, related_name='+',
                                         help_text=_('Keep empty to skip searching by leaders.'))
    template    = models.CharField(_('template'), max_length=100,
                                   choices=settings.LEPRIKON_EVENTLIST_TEMPLATES,
                                   default=settings.LEPRIKON_EVENTLIST_TEMPLATES[0][0],
                                   help_text=_('The template used to render plugin.'))

    class Meta:
        app_label = 'leprikon'

    def copy_relations(self, oldinstance):
        self.managers        = oldinstance.managers.all()
        self.event_types    = oldinstance.event_types.all()
        self.groups         = oldinstance.groups.all()
        self.age_groups     = oldinstance.age_groups.all()
        self.leaders        = oldinstance.leaders.all()

    @cached_property
    def all_managers(self):
        return list(self.managers.all())

    @cached_property
    def all_event_types(self):
        return list(self.event_types.all())

    @cached_property
    def all_age_groups(self):
        return list(self.age_groups.all())

    @cached_property
    def all_groups(self):
        return list(self.groups.all())

    @cached_property
    def all_leaders(self):
        return list(self.leaders.all())

    Group = namedtuple('Group', ('group', 'objects'))

    def render(self, context):
        school_year = (self.school_year or getattr(context.get('request'), 'school_year') or
                       SchoolYear.objects.get_current())
        events = Event.objects.filter(school_year=school_year, public=True).distinct()

        if self.all_managers:
            events = events.filter(managers__in = self.all_managers)
        if self.all_event_types:
            events = events.filter(subject_type__in = self.all_event_types)
        if self.all_age_groups:
            events = events.filter(age_groups__in = self.all_age_groups)
        if self.all_leaders:
            events = events.filter(leaders__in = self.all_leaders)
        if self.all_groups:
            events = events.filter(groups__in = self.all_groups)
            groups = self.all_groups
        elif self.all_event_types:
            groups = SubjectGroup.objects.filter(subject_types__in = self.all_event_types)
        else:
            groups = SubjectGroup.objects.all()

        context.update({
            'school_year':  school_year,
            'events':       events,
            'groups':       (
                self.Group(group = group, objects = events.filter(groups=group))
                for group in groups
            ),
        })
        return context



class FilteredEventListPlugin(CMSPlugin):
    school_year = models.ForeignKey(SchoolYear, verbose_name=_('school year'),
                                    related_name='+', blank=True, null=True)
    event_types = models.ManyToManyField(SubjectType, verbose_name=_('event types'), related_name='+',
                                         limit_choices_to={'subject_type': SubjectType.EVENT})

    class Meta:
        app_label = 'leprikon'

    def copy_relations(self, oldinstance):
        self.event_types = oldinstance.event_types.all()

    @cached_property
    def all_event_types(self):
        return list(self.event_types.all())

    def render(self, context):
        school_year = (self.school_year or getattr(context.get('request'), 'school_year') or
                       SchoolYear.objects.get_current())

        from ..forms.subjects import SubjectFilterForm
        form = SubjectFilterForm(
            subject_type_type = SubjectType.EVENT,
            subject_types = self.all_event_types,
            school_year = school_year,
            is_staff = context['request'].user.is_staff,
            data=context['request'].GET,
        )
        context.update({
            'school_year':  school_year,
            'form':         form,
            'events':      form.get_queryset(),
        })
        return context



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-04-25 14:03
from __future__ import unicode_literals

from django.db import migrations, models


def migrate_event_list_plugins(apps, schema_editor):
    from datetime import date, timedelta

    LeprikonFilteredEventListPlugin = apps.get_model('leprikon', 'LeprikonFilteredEventListPlugin')

    for plugin in LeprikonFilteredEventListPlugin.objects.all():
        plugin.event_types.add(plugin.event_type)


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0003_parents'),
    ]

    operations = [
        migrations.AddField(
            model_name='leprikonfilteredeventlistplugin',
            name='event_types',
            field=models.ManyToManyField(to='leprikon.EventType', verbose_name='event type'),
        ),
        migrations.RunPython(migrate_event_list_plugins),
        migrations.RemoveField(
            model_name='leprikonfilteredeventlistplugin',
            name='event_type',
        ),
    ]
